/**
 * Copyright (C) 2010-2012 Agrosense info@agrosense.eu
 *
 * Licensed under the Eclipse Public License - v 1.0 (the "License"); you may
 * not use this file except in compliance with the License. You may obtain a
 * copy of the License at
 *
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */
package nl.cloudfarming.client.isobus.type;

import java.io.IOException;
import java.util.List;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.xml.parsers.ParserConfigurationException;
import nl.cloudfarming.client.isobus.model.BinaryReader;
import nl.cloudfarming.client.isobus.model.GridTypeEnum;
import nl.cloudfarming.client.isobus.model.IsobusGrid;
import nl.cloudfarming.client.isobus.model.IsobusTreatmentZone;
import org.openide.filesystems.FileObject;
import org.openide.util.Exceptions;
import org.xml.sax.SAXException;

/**
 * Class which reads binary grid files There are two types of files The first
 * Grid.type == 1 the binary file is a lookup table which contains the
 * Grid.treatmentZoneCode for every square in the grid.
 *
 * the Second grid.type == 2 the binary file contains the values of the
 * ProcessDataVariable.processDataValue this is not yet supported TODO: support
 * type two
 *
 * @author Merijn Zengers
 */
public class GridReader {

    private static final Logger LOGGER = Logger.getLogger("nl.cloudfarming.client.isobus.model");
        
    private GridReader() {
    }

    /**
     * This method is only intended for binary files of Grid.type == 1 Reads the
     * Grid.treatmentZoneCode from the binary file Stores the values in the
     * returned IsobusTreatmentZoneGrid Will find the treatmentZone in the list with available
     * treatmentZones If found will add to the return list else throws an
     * assertion error 
     * Take Notice!!: the values in the file are ordered from
     * right Top to left bottom inverted from the binary
     *
     * @return a list of the read values
     * @throws IOException
     * @deprecated moving from TZN to value grid
     */
    @Deprecated
    public static IsobusTreatmentZoneGrid getTreatMentZones(FileObject gridFile, IsobusGrid grid, List<IsobusTreatmentZone> availableTreatmentZones){
        int numRows = new Long(grid.getMaximumRow()).intValue();
        int numColumns = new Long(grid.getMaximumColumn()).intValue();
        IsobusTreatmentZoneGrid treatmentZoneGrid = new IsobusTreatmentZoneGrid(numRows, numColumns);
        
        // Decrease rows and columns with one because IsobusTreatmentZoneGrid runs from 0 
        int rows = numRows - 1;
        int columns = 0;
        try(BinaryReader binaryReader = new BinaryReader(gridFile.getInputStream())){
                    
            while (binaryReader.available()) {
                long value = binaryReader.read(GridFieldEnum.TREATMENT_ZONE_CODE_TYPE);
                boolean found = false;
                for (IsobusTreatmentZone treatmentZone : availableTreatmentZones) {
                    if (treatmentZone.getTreatmentZoneCode() == value) {
                        treatmentZoneGrid.insert(rows, columns, treatmentZone);
                        found = true;
                        //Use decrease instead of increase 
                        // Our raster starts at the left top and ends at right bottom
                        // The bytes provided from the grid file starts at the left bottom and ends at the right top
                        // It has to be inverted.
                        columns++;
                        if(columns == numColumns){
                            rows--;
                            columns = 0;
                        }
                    }
                }
                assert (found);
            }
        
        } catch (NullPointerException | IOException ex) {
            LOGGER.log(Level.SEVERE, "Failed to read Treatmentzones from gridfile {0} with exception {1}", new Object[]{gridFile.getName(), ex});
            Exceptions.printStackTrace(ex);
        }
        return treatmentZoneGrid;
    }
    
    // converting from tzn grid to value grid which should be able to handle both type 1 and 2.
    public static IsobusValueGrid getValues(FileObject gridFile, IsobusGrid grid, List<IsobusTreatmentZone> availableTreatmentZones){
        int numRows = new Long(grid.getMaximumRow()).intValue();
        int numColumns = new Long(grid.getMaximumColumn()).intValue();
        IsobusValueGrid valueGrid = new IsobusValueGrid(numRows, numColumns);
        
        // Decrease rows and columns with one because IsobusTreatmentZoneGrid runs from 0 
        int rows = numRows - 1;
        int columns = 0;
        long value;
        
        IsobusTreatmentZone prototypeTZN;
        
        if (grid.getType() == GridTypeEnum.TYPE_2) {
            // find the prototype treatment zone:
            // not used yet since AgroSense uses a single process data variable 
            // with a hardcoded DDI; only validate presence for now.
            for (IsobusTreatmentZone treatmentZone : availableTreatmentZones) {
                if (treatmentZone.getTreatmentZoneCode() == grid.getTreatmentZoneCode()) {
                    prototypeTZN = treatmentZone;
                    assert prototypeTZN != null;
                }
            }
        }
        
        try (BinaryReader binaryReader = new BinaryReader(gridFile.getInputStream())){
                    
            while (binaryReader.available()) {
                
                switch (grid.getType()) {
                    case TYPE_1:
                        value = binaryReader.read(GridFieldEnum.TREATMENT_ZONE_CODE_TYPE);
                        boolean found = false;
                        for (IsobusTreatmentZone treatmentZone : availableTreatmentZones) {
                            if (treatmentZone.getTreatmentZoneCode() == value) {
                                valueGrid.insert(rows, columns, treatmentZone.getPDV().get(0).getProcessDataValue());
                                found = true;
                                //Use decrease instead of increase 
                                // Our raster starts at the left top and ends at right bottom
                                // The bytes provided from the grid file starts at the left bottom and ends at the right top
                                // It has to be inverted.
                                columns++;
                                if(columns == numColumns){
                                    rows--;
                                    columns = 0;
                                }
                            }
                        }
                        assert (found);
                        break;
                    case TYPE_2:
                        value = binaryReader.read(GridFieldEnum.TREATMENT_ZONE_VALUE_TYPE);
                        valueGrid.insert(rows, columns, (int)value);
                        //
                        columns++;
                        if(columns == numColumns){
                            rows--;
                            columns = 0;
                        }
                        break;
                    default:
                        throw new IllegalArgumentException("unsupported grid type: " + grid.getType());
                }
            }
        
        } catch (NullPointerException | IOException ex) {
            LOGGER.log(Level.SEVERE, "Failed to read Treatmentzones from gridfile {0} with exception {1}", new Object[]{gridFile.getName(), ex});
            Exceptions.printStackTrace(ex);
        }        
        return valueGrid;
    }    

}
