/*
 * Copyright (c) 2012 AgroSense.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    AgroSense - initial API and implementation and/or initial documentation
 */
package nl.cloudfarming.client.isobus.type;

import com.vividsolutions.jts.geom.Coordinate;
import com.vividsolutions.jts.geom.CoordinateSequence;
import com.vividsolutions.jts.geom.Envelope;
import com.vividsolutions.jts.geom.Geometry;
import com.vividsolutions.jts.geom.GeometryFactory;
import com.vividsolutions.jts.geom.LineString;
import com.vividsolutions.jts.geom.Point;
import com.vividsolutions.jts.geom.impl.CoordinateArraySequence;
import java.beans.PropertyChangeListener;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.logging.Logger;
import nl.cloudfarming.client.geoviewer.Geometrical;
import nl.cloudfarming.client.geoviewer.SimpleLayer;
import nl.cloudfarming.client.isobus.model.IsobusTime;
import nl.cloudfarming.client.isobus.model.PositionStatusEnum;
import org.openide.filesystems.FileObject;
import org.openide.loaders.DataNode;
import org.openide.loaders.DataObjectExistsException;
import org.openide.loaders.MultiDataObject;
import org.openide.loaders.MultiFileLoader;
import org.openide.nodes.Children;
import org.openide.nodes.Node;
import org.openide.util.Lookup;
import org.openide.util.lookup.AbstractLookup;
import org.openide.util.lookup.InstanceContent;
import org.openide.util.lookup.Lookups;
import org.openide.util.lookup.ProxyLookup;

/**
 * 
 * @author johan
 */
public class TimeLogBinaryDataObject extends MultiDataObject {
    private static final Logger LOGGER = Logger.getLogger("nl.cloudfarming.client.isobus.type.TimeLogBinaryDataObject");
    
    public static final String MIME_TYPE = "application/x-timelog-bin";
    public static final String ICON_BASE = "nl/cloudfarming/client/isobus/type/isobus_taskdata_timelog.png";
    
    private final InstanceContent ic;
    private final Lookup intern;
    private final FileObject pf;
    private final List<IsobusTime> isobusTimes;
    
    protected TimeLogBinaryDataObject(FileObject pf, MultiFileLoader loader, List<FileObject> secondary, List<IsobusTime> isobusTimes) throws DataObjectExistsException, IOException {
        super(pf, loader);
        registerEditor(MIME_TYPE, true);
        this.ic = new InstanceContent();
        this.intern = new AbstractLookup(this.ic);
        this.pf = pf;
        this.isobusTimes = isobusTimes;
        
        for (FileObject fileObject : secondary) {
            registerEntry(fileObject);
        }
    }

    @Override
    protected int associateLookup() {
        return 1;
    }
    
    @Override
    public final Lookup getLookup() {
        return new ProxyLookup(super.getLookup(), intern);
    }
    
    @Override
    protected Node createNodeDelegate() {
        if (associateLookup() >= 1) {
            DataNode node = new DataNode(this, Children.LEAF, new ProxyLookup(getLookup(), Lookups.fixed(new SimpleLayer(pf.getName()))));
            ic.add(new TimeLogGeometrical(createGeometry()));
            node.setIconBaseWithExtension(ICON_BASE);
            node.setDisplayName(pf.getName());
            return node;
        }
        return super.createNodeDelegate();
    }

    private Geometry createGeometry() {
        List<Coordinate> coords = new ArrayList<>();
        for (IsobusTime time : isobusTimes) {
            PositionStatusEnum status = time.getPTN().getPositionStatus();

            double lon = time.getPTN().getPositionEast().doubleValue();
            double lat = time.getPTN().getPositionNorth().doubleValue();

            if (status != null && status != PositionStatusEnum.ERROR && status != PositionStatusEnum.NO_GPS_FIX) {
                coords.add(new Coordinate(lon, lat));
            }
        }
        Coordinate[] coordArray = coords.toArray(new Coordinate[coords.size()]);
        CoordinateSequence coordSequence = new CoordinateArraySequence(coordArray);
        LineString line = new LineString(coordSequence, new GeometryFactory());
        return line;
    }

    public List<IsobusTime> getIsobusTimes() {
        return Collections.unmodifiableList(isobusTimes);
    }
    
    private class TimeLogGeometrical implements Geometrical {
        private final Geometry geometry;

        public TimeLogGeometrical(Geometry geometry) {
            this.geometry = geometry;
        }
        
        @Override
        public Geometry getGeometry() {
            return geometry;
        }

        @Override
        public Point getCentroid() {
            return geometry == null ? null : geometry.getCentroid();
        }

        @Override
        public Geometry getBoundingBox() {
            if (geometry == null) { return null; }
            Geometry envelope = geometry.getEnvelope();
            return envelope.getNumPoints() < 4 ? null : envelope;
        }

        @Override
        public Object getRenderObject(Envelope envelope) {
            return null;
        }

        @Override
        public String getTooltipText() {
            return pf.getName();
        }

        @Override
        public String getIconPath() {
            return null;
        }

        @Override
        public String getIconLabel() {
            return null;
        }

        @Override
        public void addPropertyChangeListener(PropertyChangeListener listener) {
        }

        @Override
        public void removePropertyChangeListener(PropertyChangeListener listener) {
        }
    }
}
