/*
 * Copyright (c) 2012 AgroSense.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    AgroSense - initial API and implementation and/or initial documentation
 */
package nl.cloudfarming.client.isobus.type;

import java.io.FileNotFoundException;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;
import java.util.logging.Level;
import java.util.logging.Logger;
import java.util.regex.Pattern;
import javax.xml.parsers.ParserConfigurationException;
import nl.cloudfarming.client.isobus.model.IsobusTaskData;
import nl.cloudfarming.client.isobus.model.IsobusTime;
import org.openide.awt.ActionID;
import org.openide.awt.ActionReference;
import org.openide.awt.ActionReferences;
import org.openide.filesystems.FileObject;
import org.openide.filesystems.MIMEResolver;
import org.openide.loaders.DataObject;
import org.openide.loaders.DataObjectExistsException;
import org.openide.loaders.FileEntry;
import org.openide.loaders.MultiDataObject;
import org.openide.loaders.MultiDataObject.Entry;
import org.openide.loaders.MultiFileLoader;
import org.openide.util.Exceptions;
import org.openide.util.NbBundle;
import org.xml.sax.SAXException;

/**
 *
 * @author johan
 */
@DataObject.Registration(
    mimeType = TimeLogBinaryDataObject.MIME_TYPE,
iconBase = TimeLogBinaryDataObject.ICON_BASE,
displayName = "#LBL_TimeLogBinary_LOADER",
position = 300)
@MIMEResolver.Registration(
    displayName = "#LBL_TimeLogBinary_LOADER",
resource="TimeLogBinaryResolver.xml")
@ActionReferences(value = {
    @ActionReference(id =
    @ActionID(category = "System", id = "org.openide.actions.OpenAction"), path = "Loaders/application/x-timelog-bin/Actions", position = 100, separatorAfter = 200),
    @ActionReference(id =
    @ActionID(category = "System", id = "org.openide.actions.ToolsAction"), path = "Loaders/application/x-timelog-bin/Actions", position = 1300)})
@NbBundle.Messages({
    "LBL_TimeLogBinary_LOADER=Files of TimeLogBinary"
})
public class TimeLogBinaryFileLoader extends MultiFileLoader {
    private static final Logger LOGGER = Logger.getLogger("nl.cloudfarming.client.isobus.type.TimeLogBinaryFileLoader");
    
    public static final String TIMELOG_BIN_PATTERN = "(?i)tlg[0-9]{5}\\.bin";
    public static final String TIMELOG_XML_PATTERN = "(?i)tlg[0-9]{5}\\.xml";
    public static final String TASKDATA_PATTERN = "(?i)taskdata\\.xml";

    public TimeLogBinaryFileLoader() {
        super(TimeLogBinaryDataObject.class.getName());
    }

    @Override
    protected FileObject findPrimaryFile(FileObject fo) {
        // never recognize folders.
        if (fo.isFolder()) {
            return null;
        }
        
        Pattern timeLogBinPattern = Pattern.compile(TIMELOG_BIN_PATTERN);
        Pattern timeLogXmlPattern = Pattern.compile(TIMELOG_XML_PATTERN);

        if (timeLogBinPattern.matcher(fo.getNameExt()).find()) {
            return fo;
        } else if (timeLogXmlPattern.matcher(fo.getNameExt()).find()) {
            // find timelog binary with same name:
            for (FileObject child : fo.getParent().getChildren()) {
                if (!child.equals(fo)) {
                    if (timeLogBinPattern.matcher(child.getNameExt()).find() 
                            && child.getName().equalsIgnoreCase(fo.getName())) {
                        return child;
                    }
                }
            }
        }
        // not recognized:
        return null;
    }

    @Override
    protected String actionsContext() {
        StringBuilder builder = new StringBuilder("Loaders/");
        builder.append(TimeLogBinaryDataObject.MIME_TYPE);
        builder.append("/Actions/");
        return builder.toString();
    }

    @Override
    protected MultiDataObject createMultiObject(FileObject primaryFile) throws DataObjectExistsException, IOException {
        List<FileObject> secondary = new ArrayList<>();
        List<IsobusTime> times = new ArrayList<>();
        
        Pattern timeLogXmlPattern = Pattern.compile(TIMELOG_XML_PATTERN);
        Pattern taskDataPattern = Pattern.compile(TASKDATA_PATTERN);
        
        FileObject taskDataFile = null;
        
        for (FileObject child : primaryFile.getParent().getChildren()) {
            if (!child.equals(primaryFile)) {
                if (taskDataPattern.matcher(child.getNameExt()).find()) {
                    // required to load data but don't claim it as a secondary file:
                    taskDataFile = child;
                } else if (timeLogXmlPattern.matcher(child.getNameExt()).find()
                        && child.getName().equalsIgnoreCase(primaryFile.getName())) {
                    secondary.add(child);
                }
            }
        }
        
        if (taskDataFile != null) {
            try {
                TaskDataObject taskDataObject = (TaskDataObject)DataObject.find(taskDataFile);
                IsobusTaskData isobusTaskData = taskDataObject.getIsobusTaskData();
                TimeLogReader reader = new TimeLogReader(primaryFile.getName(), primaryFile.getParent(), isobusTaskData);
                times.addAll(reader.getTimes());
            } catch (ParserConfigurationException | SAXException | FileNotFoundException ex) {
                Exceptions.printStackTrace(ex);
                LOGGER.log(Level.SEVERE, "Exception while loading timelog data: {0}", ex);
            }
        } else {
            LOGGER.severe("No taskdata.xml file found");
        }

        return new TimeLogBinaryDataObject(primaryFile, this, secondary, times);
    }

    @Override
    protected Entry createPrimaryEntry(MultiDataObject obj, FileObject primaryFile) {
        Entry entry = new FileEntry(obj, primaryFile);
        LOGGER.log(Level.FINEST, "Creating primary entry for {0}", primaryFile.getNameExt());
        return entry;
    }

    @Override
    protected Entry createSecondaryEntry(MultiDataObject obj, FileObject secondaryFile) {
        FileEntry entry = new FileEntry(obj, secondaryFile);
        LOGGER.log(Level.FINEST, "Creating secondary entry for {0}", secondaryFile.getNameExt());
        return entry;
    }
    
}
