/*
 * Copyright (c) 2012 AgroSense.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    AgroSense - initial API and implementation and/or initial documentation
 */
package nl.cloudfarming.client.isobus.type;

import java.awt.BorderLayout;
import java.math.BigInteger;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import javax.swing.JScrollPane;
import javax.swing.table.AbstractTableModel;
import nl.cloudfarming.client.isobus.model.DDI;
import nl.cloudfarming.client.isobus.model.DataLogValue;
import nl.cloudfarming.client.isobus.model.DeviceElement;
import nl.cloudfarming.client.isobus.model.IsobusTime;
import nl.cloudfarming.client.isobus.model.PositionStatusEnum;
import org.jdesktop.swingx.JXTable;
import org.jdesktop.swingx.renderer.DefaultTableRenderer;
import org.jdesktop.swingx.renderer.StringValues;
import org.openide.util.NbBundle;

/**
 *
 * @author johan
 */
public class TimeLogBinaryTablePanel extends javax.swing.JPanel {

    /**
     * Creates new form TimeLogBinaryTablePanel
     */
    public TimeLogBinaryTablePanel(List<IsobusTime> times) {
//        initComponents();
        setLayout(new BorderLayout());        
        JXTable table = new JXTable(new TimeLogTableModel(times));
        JScrollPane scrollPane = new JScrollPane(table);
        
        table.setFillsViewportHeight(true);
        table.setHorizontalScrollEnabled(true);
        table.setColumnControlVisible(true);

        // prefer basic toString() over the default formatters used by SwingX (where details are lost due to rounding, no time, ...)
        table.setDefaultRenderer(Date.class, new DefaultTableRenderer(StringValues.TO_STRING));
        table.setDefaultRenderer(Number.class, new DefaultTableRenderer(StringValues.TO_STRING));
        
        add(scrollPane, BorderLayout.CENTER);
    }
    
    @NbBundle.Messages({
        "TimeLogTable.ColumnName.Rownum=#",
        "TimeLogTable.ColumnName.Date=Date",
        "TimeLogTable.ColumnName.Latitude=Latitude",
        "TimeLogTable.ColumnName.Longitude=Longitude",
        "TimeLogTable.ColumnName.GPS_Status=GPS status",
    })
    private static class TimeLogTableModel extends AbstractTableModel {
        public static final String[] BASE_COLUMN_NAMES = new String[] {
            Bundle.TimeLogTable_ColumnName_Rownum(),
            Bundle.TimeLogTable_ColumnName_Date(),
            Bundle.TimeLogTable_ColumnName_Latitude(),
            Bundle.TimeLogTable_ColumnName_Longitude(),
            Bundle.TimeLogTable_ColumnName_GPS_Status(),
        };
        public static final Class[] BASE_COLUMN_CLASSES = new Class[] {Integer.class, Date.class, BigInteger.class, BigInteger.class, String.class};
        public static final int BASE_COLUMN_COUNT = BASE_COLUMN_NAMES.length;
        static {
            assert BASE_COLUMN_CLASSES.length == BASE_COLUMN_COUNT;
        }
        
        private final List<IsobusTime> times;     

        private Object[][] rowData;
        private String[] columnNames;
        private Class[] columnClasses;
        // cache DDIs per DLV column:
        private DDI[] ddiLookup;
        private int rowCount;
        private int colCount;
        private int dlvCount;
        
        public TimeLogTableModel(List<IsobusTime> times) {
            this.times = times;
            initColumns();
            initRowData();
        }
        
        private void initColumns() {
            List<DataLogValue> dlvs = new ArrayList<>();

            // using first record as template for columns;
            // assuming TimeLogReader has created 0-valued DLV elements for any indices not present in the first binary record.
            if (!times.isEmpty()) {
                IsobusTime time = times.iterator().next();
                if (time.getDLV() != null) {
                    dlvs.addAll(time.getDLV());
                }
            }

            dlvCount = dlvs.size();
            colCount = BASE_COLUMN_COUNT + dlvCount;
            ddiLookup = new DDI[dlvCount];
            columnNames = new String[colCount];
            columnClasses = new Class[colCount];
            
            System.arraycopy(BASE_COLUMN_NAMES, 0, columnNames, 0, BASE_COLUMN_COUNT);
            System.arraycopy(BASE_COLUMN_CLASSES, 0, columnClasses, 0, BASE_COLUMN_COUNT);

            for(int j = 0; j < dlvCount; j++) {
                DataLogValue dlv = dlvs.get(j);
                //TODO:  add full names (DDI description, DET designator) to tooltips.
                String detStr = dlv.getDeviceElementIdRef() == null ? "unknown DET" : ((DeviceElement)dlv.getDeviceElementIdRef()).getId();
                String ddiStr;
                byte[] bytes = dlv.getProcessDataDDI();
                DDI ddi = DDI.findById(bytes);
                if (ddi != null) {
                    ddiLookup[j] = ddi;
                    ddiStr = ddi.name();
                } else {
                    int ddiCode = (bytes[0] << 8) + (bytes[1] & 255);
                    ddiStr = "unknown DDI_" + ddiCode;
                }

                columnNames[BASE_COLUMN_COUNT + j] = detStr + "/" + ddiStr;
                columnClasses[BASE_COLUMN_COUNT + j] = Double.class;
            }
        }
        
        private void initRowData() {
            rowCount = times.size();
            rowData = new Object[rowCount][];

            for(int i = 0; i < rowCount; i++) {
                int col = 0;
                IsobusTime time = times.get(i);
                Object[] rec = new Object[colCount];
                rec[col] = Integer.valueOf(i); // add rownum to be able to restore original datafile order

                rec[++col] = time.getTimeStart();

                rec[++col] = time.getPTN().getPositionNorth();
                rec[++col] = time.getPTN().getPositionEast();
                PositionStatusEnum sts = time.getPTN().getPositionStatus();
                rec[++col] = sts == null ? "" : (sts.value() + ": " + sts.toString());

                assert time.getDLV().size() == dlvCount;
                
                if (dlvCount > 0) {
                    List<DataLogValue> dlvs = time.getDLV();
                    
                    for(int j = 0; j < dlvCount; j++) {
                        DDI ddi = ddiLookup[j];
                        int val = dlvs.get(j).getProcessDataValue();
                        // show raw int values for now when DDI is not available:
                        rec[BASE_COLUMN_COUNT + j] = Double.valueOf(ddi == null ? (double)val : ddi.toImportValue(val));
                    }          
                }
                rowData[i] = rec;
            }            
        }
        
        @Override
        public int getRowCount() {
            return rowCount;
        }

        @Override
        public int getColumnCount() {
            return colCount;
        }

        @Override
        public Object getValueAt(int rowIndex, int columnIndex) {
            return rowData[rowIndex][columnIndex];
        }

        @Override
        public String getColumnName(int column) {
            return columnNames[column];
        }

        @Override
        public Class<?> getColumnClass(int columnIndex) {
            return columnClasses[columnIndex];
        }
    }

    /**
     * This method is called from within the constructor to initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is always
     * regenerated by the Form Editor.
     */
    @SuppressWarnings("unchecked")
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {

        javax.swing.GroupLayout layout = new javax.swing.GroupLayout(this);
        this.setLayout(layout);
        layout.setHorizontalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGap(0, 400, Short.MAX_VALUE)
        );
        layout.setVerticalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGap(0, 300, Short.MAX_VALUE)
        );
    }// </editor-fold>//GEN-END:initComponents
    // Variables declaration - do not modify//GEN-BEGIN:variables
    // End of variables declaration//GEN-END:variables
}
